#include "StdAfx.h"
#include "SafeSocketHandle.h"

using namespace Adaptive::Win32;
using namespace Adaptive::Network;
using namespace Adaptive::Network::Sockets;

/*--------------------------------------------------------------------------------
 * Constructor / Dispose / Destructor Methods
 *------------------------------------------------------------------------------*/
#pragma region Constructor / Dispose / Destructor Methods
// <summary>
// Initializes a new instance of the class.
// </summary>
// <remarks>
// This is the default constructor.
// </remarks>
SafeSocketHandle::SafeSocketHandle(): SafeHandle(IntPtr::Zero, false)
{
}
// <summary>
// Initializes a new instance of the class to the specified handle value.
// </summary>
// <param name="originalHandle">The original handle value.</param>
SafeSocketHandle::SafeSocketHandle(IntPtr originalHandle) : SafeHandle(originalHandle, true)
{
}
// <summary>
// Initializes a new instance of the class to the specified handle value.
// </summary>
// <param name="originalHandle">The original handle value.</param>
SafeSocketHandle::SafeSocketHandle(IntPtr^ originalHandle) : SafeHandle(*originalHandle, true)
{
}
// Initializes a new instance of the class to the specified handle value.
SafeSocketHandle::SafeSocketHandle(System::Net::Sockets::Socket^ netSocket) : SafeHandle(IntPtr::Zero, false)
{
	if (netSocket != nullptr)
	{
		SetHandle(netSocket->Handle);
	}
}
// <summary>
// Initializes a new instance of the class to the specified unmanaged handle value.
// </summary>
// <param name="originalHandle">The original handle value.</param>
SafeSocketHandle::SafeSocketHandle(SOCKET originalHandle) : SafeHandle(IntPtr::Zero, true)
{
	IntPtr^ handle;

	handle = gcnew IntPtr((int)originalHandle);
	SetHandle(*handle);
}
#pragma endregion

/*--------------------------------------------------------------------------------
 * Public Properties
 *------------------------------------------------------------------------------*/
#pragma region Public Properties
// <summary>
// Gets the underlying handle value.
// </summary>
// <value>
// An <see cref="IntPtr"/> value indicating the underlying handle.
// </value>
bool SafeSocketHandle::IsInvalid::get()
{
	return (IsClosed || handle == IntPtr::Zero);
}
// <summary>
// Gets a value indicating whether the underlying handle is invalid.
// </summary>
// <value>
// <b>true</b> if the underlying handle value is an invalid handle or null value;
// otherwise, <b>false</b>.
// </value>
IntPtr SafeSocketHandle::HandleValue::get()
{
	return handle;
}
#pragma endregion

/*--------------------------------------------------------------------------------
 * Protected Methods / Event Methods
 *------------------------------------------------------------------------------*/
#pragma region Protected Methods / Event Methods
// <summary>
// Releases the underlying handle value.
// </summary>
bool SafeSocketHandle::ReleaseHandle()
{
	SOCKET handleValue = NULL;
	bool returnValue = false;

	if (!IsInvalid)
	{
		handleValue = (SOCKET)(handle.ToInt32());
		try
		{
			::closesocket(handleValue);
		}
		catch(...)
		{
		}
		SetHandle(IntPtr::Zero);
		returnValue = true;
	}
	return returnValue;
}
#pragma endregion
