/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework
 * Version 2.10
 * 
 * Stored Procedure Result Type Enumeration
 *      Lists the types of results that may occur from executing a stored procedure.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;

namespace Adaptive.Foundation.Data
{
    /// <summary>
    /// Lists the types of results that may occur from executing a stored procedure.
    /// </summary>
    /// <remarks>
    /// This is used on the <see cref="StoredProcedureResult"/> class to determine where
    /// the results of the procedure call are located, as well as in the <see cref="StoredProcedureBase"/> class
    /// when executing in order to determine how the procedure is executed, and in which property of 
    /// <b>StoredProcedureResult</b> the results are placed.
    /// </remarks>
    public enum StoredProcedureResultType
    {
        /// <summary>
        /// Indicates that the stored procedure is execute only and returns nothing.
        /// </summary>
        None = 0,
        /// <summary>
        /// Indicates that the stored procedure returns a resultset as a data table.
        /// </summary>
        DataTable = 1,
        /// <summary>
        /// Indicates that the stored procedure returns nultiple resultsets in a data set.
        /// </summary>
        DataSet = 2,
        /// <summary>
        /// Indicates that the stored procedure returns the number of rows affected by the query.
        /// </summary>
        RowCount = 3

    }
}