/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework Class Library
 * Version 2.00
 *
 * Authentication Require Event Arguments Class
 *		Provides an event arguments definition for authentication events.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Text;

namespace Adaptive.Foundation
{
    /// <summary>
    /// Provides an event arguments definition for authentication events.
    /// </summary>
    public sealed class AuthenticationRequiredEventArgs : EventArgs
    {
        /*--------------------------------------------------------------------------------
         * Private Member Declarations
         *------------------------------------------------------------------------------*/
        #region Private Member Declarations
        /// <summary>
        /// Password value.
        /// </summary>
        private string _password = string.Empty;
        /// <summary>
        /// Cancel flag.
        /// </summary>
        private bool _cancel;
        #endregion

        /*--------------------------------------------------------------------------------
		 * Constructor / Dispose / Destructor Methods
		 *------------------------------------------------------------------------------*/
        #region Constructor / Dispose / Destructor Methods
        /// <summary>
        /// Initializes a new instance of the class.
        /// </summary>
        /// <remarks>
        /// This is the default constructor.
        /// </remarks>
        public AuthenticationRequiredEventArgs()
        {
        }
        #endregion

        /*--------------------------------------------------------------------------------
		 * Public Properties
		 *------------------------------------------------------------------------------*/
        #region Public Properties
        /// <summary>
        /// Gets or sets a password value to use in an authentication process.
        /// </summary>
        /// <value>
        /// A user-supplied value.
        /// </value>
        public string Password
        {
            get { return _password; }
            set { _password = value; }
        }
        /// <summary>
        /// Gets or sets a value indicating whether to cancel the current operation.
        /// </summary>
        /// <value>
        /// <b>true</b> to cancel the current operation; otherwise, <b>false</b>.
        /// </value>
        public bool Cancel
        {
            get { return _cancel; }
            set { _cancel = value; }
        }
        #endregion
    }
}
