﻿/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework
 * Version 2.10
 * 
 * POP3 Negative Response Code Class
 *      Represents the negative response code for the POP3 protocol.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;
using Adaptive.Foundation;
using Adaptive.Foundation.Network;
using Adaptive.Foundation.Network.Cnr;
using Adaptive.Network.Sockets;

namespace Adaptive.Foundation.Network.Pop3
{
    /// <summary>
    /// Represents the negative response code for the POP3 protocol.
    /// </summary>
    public sealed class NegativeResponseCode : CnrErrorCode<string>
    {
        /*--------------------------------------------------------------------------------
         * Private Constants Declarations
         *------------------------------------------------------------------------------*/
        #region Private Constants Declarations

        private const string TEXT = "-ERR";

        #endregion

        /*--------------------------------------------------------------------------------
         * Constructor / Dispose / Destructor Methods
         *------------------------------------------------------------------------------*/
        #region Constructor / Dispose / Destructor Methods
        /// <summary>
        /// Initializes a new instance of the class.
        /// </summary>
        /// <remarks>
        /// This is the default constructor.
        /// </remarks>
        public NegativeResponseCode()
            : base(TEXT)
        {
        }
        #endregion

        /*--------------------------------------------------------------------------------
         * Public Methods / Functions
         *------------------------------------------------------------------------------*/
        #region Public Methods / Functions
        /// <summary>
        /// Clears the contents of the object instance without disposing of the instance.
        /// </summary>
        public override void Clear()
        {
            base.Clear();
        }
        /// <summary>
        /// Renders the error code as text.
        /// </summary>
        /// <returns>
        /// The error code as a text string.
        /// </returns>
        public override string Render()
        {
            return base.Render(new Pop3Renderer());
        }
        #endregion


    }
}