/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework Class Library
 * Version 2.00
 * 
 * Cryptographic Result Type Enumeration
 *      Lists the cryptographic results that may occur.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;

namespace Adaptive.Foundation.Security
{
    /// <summary>
    /// Lists the cryptographic results that may occur.
    /// </summary>
    public enum CryptographicResultType
    {
        /// <summary>
        /// Indicates no operation was performed.
        /// </summary>
        None = 0,
        /// <summary>
        /// Indicates the operation was successful.
        /// </summary>
        Success = 1,
        /// <summary>
        /// Indicates the parameters were invalid.
        /// </summary>
        InvalidParameters = 2,
        /// <summary>
        /// Indicates an invalid algorithm was specified to an object instance.
        /// </summary>
        InvalidAlgorithm = 3,
        /// <summary>
        /// Indicates a cryptographic exception occurred.
        /// </summary>
        CryptographicException = 4,
        /// <summary>
        /// Indicates an exception occurred from reading the source data.
        /// </summary>
        SourceDataException = 5,
        /// <summary>
        /// Indicates an unknown exception occurred during operation.
        /// </summary>
        UnknownException = 6
    }
}