/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework Class Library
 * Version 2.00
 * 
 * Symmetric Key Request Event Arguments Class
 *      Provides an event arguments class for events relating to symmetric algorithm key requests.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;

namespace Adaptive.Foundation.Security
{
    /// <summary>
    /// Provides an event arguments class for events relating to symmetric algorithm key requests.
    /// </summary>
    public class SymmetricKeyRequestEventArgs : KeyRequestEventArgs
    {
        /*--------------------------------------------------------------------------------
         * Private Member Declarations
         *------------------------------------------------------------------------------*/
        #region Private Member Declarations
        /// <summary>
        /// IV value to return to requestor.
        /// </summary>
        private byte[] _ivValue;
        #endregion

        /*--------------------------------------------------------------------------------
         * Constructor / Dispose / Destructor Methods
         *------------------------------------------------------------------------------*/
        #region Constructor / Dispose / Destructor Methods
        /// <summary>
        /// Initializes a new instance of the class.
        /// </summary>
        /// <remarks>
        /// This is the default constructor.
        /// </remarks>
        public SymmetricKeyRequestEventArgs()
            : base()
        {
        }
        #endregion

        /*--------------------------------------------------------------------------------
         * Public Properties
         *------------------------------------------------------------------------------*/
        #region Public Properties
        /// <summary>
        /// Gets or sets the IV value to be returned to the caller.
        /// </summary>
        /// <value>
        /// A byte array containing the IV value to use for encryption or decryption operations.
        /// </value>
        public byte[] IVValue
        {
            get
            {
                return _ivValue;
            }
            set
            {
                _ivValue = value;
            }
        }
        #endregion
    }
}